@@ -3,10 +3,17 @@
 //! Shallow wrapper around an underlying `[T; N]`, which panics if the
 //! array bounds are exceeded.
 //!
+//! ## Optional features
+//!
+//! ### `serde`
+//!
+//! When this optional dependency is enabled, `StackVec` implements the `serde::Serialize` and
+//! `serde::Deserialize` traits.
+//!
 //! ## Rust Version
 //!
 //! This version of `stack-buf` requires Rust 1.51 or later.
 
 mod vec;
 
-pub use vec::StackVec;
+pub use vec::{IntoIter, StackVec};
