@@ -2,7 +2,10 @@ use std::borrow::{Borrow, BorrowMut};
 use std::cmp::Ordering;
 use std::hash::{Hash, Hasher};
 use std::mem::{ManuallyDrop, MaybeUninit};
-use std::ops::{Deref, DerefMut};
+use std::ops::{
+    Deref, DerefMut, Index, IndexMut, Range, RangeFrom, RangeFull, RangeInclusive, RangeTo,
+    RangeToInclusive,
+};
 use std::{fmt, ptr, slice};
 
 type Size = u32;
@@ -702,6 +705,269 @@ impl<const N: usize> std::io::Write for StackVec<u8, N> {
     }
 }
 
+impl<const N: usize> fmt::Write for StackVec<u8, N> {
+    #[inline]
+    fn write_str(&mut self, s: &str) -> fmt::Result {
+        self.copy_from_slice(s.as_bytes());
+        Ok(())
+    }
+}
+
+impl<T, const N: usize> Index<usize> for StackVec<T, N> {
+    type Output = T;
+
+    #[inline]
+    fn index(&self, index: usize) -> &Self::Output {
+        &self.as_slice()[index]
+    }
+}
+
+impl<T, const N: usize> IndexMut<usize> for StackVec<T, N> {
+    #[inline]
+    fn index_mut(&mut self, index: usize) -> &mut Self::Output {
+        &mut self.as_mut_slice()[index]
+    }
+}
+
+impl<T, const N: usize> Index<RangeFull> for StackVec<T, N> {
+    type Output = [T];
+
+    #[inline]
+    fn index(&self, _index: RangeFull) -> &Self::Output {
+        self.as_slice()
+    }
+}
+
+impl<T, const N: usize> IndexMut<RangeFull> for StackVec<T, N> {
+    #[inline]
+    fn index_mut(&mut self, _index: RangeFull) -> &mut Self::Output {
+        self.as_mut_slice()
+    }
+}
+
+macro_rules! impl_range_index {
+    ($idx_ty: ty) => {
+        impl<T, const N: usize> Index<$idx_ty> for StackVec<T, N> {
+            type Output = [T];
+
+            #[inline]
+            fn index(&self, index: $idx_ty) -> &Self::Output {
+                &self.as_slice()[index]
+            }
+        }
+
+        impl<T, const N: usize> IndexMut<$idx_ty> for StackVec<T, N> {
+            #[inline]
+            fn index_mut(&mut self, index: $idx_ty) -> &mut Self::Output {
+                &mut self.as_mut_slice()[index]
+            }
+        }
+    };
+    ($($idx_ty: ty),+ $(,)?) => {
+        $(impl_range_index!($idx_ty);)+
+    }
+}
+
+impl_range_index!(
+    Range<usize>,
+    RangeFrom<usize>,
+    RangeInclusive<usize>,
+    RangeTo<usize>,
+    RangeToInclusive<usize>,
+);
+
+/// Iterate the `StackVec` with references to each element.
+///
+/// ```
+/// use stack_buf::StackVec;
+///
+/// let vec = StackVec::from([1, 2, 3]);
+///
+/// for ele in &vec {
+///     // ...
+/// }
+/// ```
+impl<'a, T, const N: usize> IntoIterator for &'a StackVec<T, N> {
+    type Item = &'a T;
+    type IntoIter = slice::Iter<'a, T>;
+
+    #[inline]
+    fn into_iter(self) -> Self::IntoIter {
+        self.iter()
+    }
+}
+
+/// Iterate the `StackVec` with mutable references to each element.
+///
+/// ```
+/// use stack_buf::StackVec;
+///
+/// let mut vec = StackVec::from([1, 2, 3]);
+///
+/// for ele in &mut vec {
+///     // ...
+/// }
+/// ```
+impl<'a, T, const N: usize> IntoIterator for &'a mut StackVec<T, N> {
+    type Item = &'a mut T;
+    type IntoIter = slice::IterMut<'a, T>;
+
+    #[inline]
+    fn into_iter(self) -> Self::IntoIter {
+        self.iter_mut()
+    }
+}
+
+/// Iterate the `StackVec` with each element by value.
+///
+/// The vector is consumed by this operation.
+///
+/// ```
+/// use stack_buf::StackVec;
+///
+/// for ele in StackVec::from([1, 2, 3]) {
+///     // ...
+/// }
+/// ```
+impl<T, const N: usize> IntoIterator for StackVec<T, N> {
+    type Item = T;
+    type IntoIter = IntoIter<T, N>;
+
+    #[inline]
+    fn into_iter(self) -> Self::IntoIter {
+        IntoIter {
+            vec: self,
+            index: 0,
+        }
+    }
+}
+
+/// An iterator that consumes a `StackVec` and yields its items by value.
+///
+/// Returned from [`StackVec::into_iter()`].
+pub struct IntoIter<T, const N: usize> {
+    vec: StackVec<T, N>,
+    index: usize,
+}
+
+impl<T, const N: usize> Iterator for IntoIter<T, N> {
+    type Item = T;
+
+    #[inline]
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.index == self.vec.len() {
+            None
+        } else {
+            unsafe {
+                let index = self.index;
+                self.index = index + 1;
+                Some(ptr::read(self.vec.as_mut_ptr().add(index)))
+            }
+        }
+    }
+
+    #[inline]
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        let len = self.vec.len() - self.index;
+        (len, Some(len))
+    }
+}
+
+impl<T, const N: usize> DoubleEndedIterator for IntoIter<T, N> {
+    #[inline]
+    fn next_back(&mut self) -> Option<Self::Item> {
+        if self.index == self.vec.len() {
+            None
+        } else {
+            unsafe {
+                let new_len = self.vec.len() - 1;
+                self.vec.set_len(new_len);
+                Some(ptr::read(self.vec.as_mut_ptr().add(new_len)))
+            }
+        }
+    }
+}
+
+impl<T, const N: usize> ExactSizeIterator for IntoIter<T, N> {}
+
+impl<T, const N: usize> Drop for IntoIter<T, N> {
+    #[inline]
+    fn drop(&mut self) {
+        let index = self.index;
+        let len = self.vec.len();
+        unsafe {
+            self.vec.set_len(0);
+            let elements = slice::from_raw_parts_mut(self.vec.as_mut_ptr().add(index), len - index);
+            ptr::drop_in_place(elements);
+        }
+    }
+}
+
+impl<T: fmt::Debug, const N: usize> fmt::Debug for IntoIter<T, N> {
+    #[inline]
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.debug_list().entries(&self.vec[self.index..]).finish()
+    }
+}
+
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Deserializer, Serialize, Serializer};
+
+#[cfg(feature = "serde")]
+impl<T: Serialize, const N: usize> Serialize for StackVec<T, N> {
+    #[inline]
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.collect_seq(self.as_slice())
+    }
+}
+
+#[cfg(feature = "serde")]
+impl<'de, T: Deserialize<'de>, const N: usize> Deserialize<'de> for StackVec<T, N> {
+    #[inline]
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        use serde::de::{Error, SeqAccess, Visitor};
+        use std::marker::PhantomData;
+
+        struct StackVecVisitor<'de, T: Deserialize<'de>, const N: usize>(
+            PhantomData<(&'de (), [T; N])>,
+        );
+
+        impl<'de, T: Deserialize<'de>, const N: usize> Visitor<'de> for StackVecVisitor<'de, T, N> {
+            type Value = StackVec<T, N>;
+
+            fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                write!(formatter, "an array with no more than {} items", N)
+            }
+
+            #[inline]
+            fn visit_seq<SA>(self, mut seq: SA) -> Result<Self::Value, SA::Error>
+            where
+                SA: SeqAccess<'de>,
+            {
+                let mut values = StackVec::<T, N>::new();
+
+                while let Some(value) = seq.next_element()? {
+                    if values.is_full() {
+                        return Err(SA::Error::invalid_length(N + 1, &self));
+                    }
+
+                    values.push(value);
+                }
+
+                Ok(values)
+            }
+        }
+
+        deserializer.deserialize_seq(StackVecVisitor::<T, N>(PhantomData))
+    }
+}
+
 /// Creates a [`StackVec`] containing the arguments.
 ///
 /// `stack_vec!` allows `StackVec`s to be defined with the same syntax as array expressions.
